/*
	
	FAT
	===
	
	Program that takes appropriate actions based on the commands selected by the user
	
	Commands include:
		* List directories and files (at current directory)
		* Change directory (to a folder specified by user)
		* Go to a directory one level up (like  cd .. in Unix)
		* Go to Root directory
		* Get current working directory (like pwd in Unix)
		* Create directory
		* Copy file (from PC/DSP to disk)
		* Read file (from disk to PC/DSP)
		* Erase file
		* Searches for a file/directory

	Note: FAT entries will be automatically read by DSP. Separate commands are not 
		  required for this.
	
*/

#include "include.h"
#include "FAT16.h"
#include <def21369.h>
#include <cdef21369.h>
#include <sru.h>
#include <signal.h>

volatile int command_fat=0;

void main()
{

    InitPLL_SDRAM();
    init_SRU();
    mmc_init();
	
	// As initialization is done, baud rate could be increased.
	*pSPIBAUD = 80; // BAUDR = 150 => SPI Baud Rate = 331.776 / (8*40) = 1.0368 MHz
	
	// Set the block length - CMD16
	mmc_set_block_length(512);
	
	
	int* dest1;    
    dest1 = (int *)malloc(514 * sizeof(int));
	
    mmc_read_data(0x3e000,512,dest1);
        	
	GetFATInfo();
	
    // Continously refresh the screen with the options available to the user till the user wishes
    // to exit from the program
    while(1)
    {
        if(command_fat!=0)
        {
    		CommandInterpreter(command_fat);
    		command_fat=0;
        }
    }

}


void InitPLL_SDRAM(){

/********************************************************************************************/

int i, pmctlsetting;

//Change this value to optimize the performance for quazi-sequential accesses (step > 1)
#define SDMODIFY 1

    pmctlsetting= *pPMCTL;
    pmctlsetting &= ~(0xFF); //Clear

    // CLKIN= 24.576 MHz, Multiplier= 27, Divisor= 2, CCLK_SDCLK_RATIO 2.5
    // Core clock = (24.576 MHz * 27) /2 = 331.776 MHz
    pmctlsetting= SDCKR2_5|PLLM27|INDIV|DIVEN;
    *pPMCTL= pmctlsetting;
    pmctlsetting|= PLLBP;
    pmctlsetting^= DIVEN;
    *pPMCTL= pmctlsetting;

    //Wait for around 4096 cycles for the pll to lock.
    for (i=0; i<4096; i++)
          asm("nop;");

    *pPMCTL ^= PLLBP;       //Clear Bypass Mode
    *pPMCTL |= (CLKOUTEN);  //and start clkout


    // Programming SDRAM control registers and enabling SDRAM read optimization
    // CCLK_SDCLK_RATIO= 2.5
    // RDIV = ((f SDCLK X t REF )/NRA) - (tRAS + tRP )
    // (133*(10^6)*64*(10^-3)/4096) - (7+3) = 2068

    *pSDRRC= (0x814)|(SDMODIFY<<17)|SDROPT;

    //===================================================================
    //
    // Configure SDRAM Control Register (SDCTL) for PART MT48LC4M32B2
    //
    //  SDCL3  : SDRAM CAS Latency= 3 cycles
    //  DSDCLK1: Disable SDRAM Clock 1
    //  SDPSS  : Start SDRAM Power up Sequence
    //  SDCAW8 : SDRAM Bank Column Address Width= 8 bits
    //  SDRAW12: SDRAM Row Address Width= 12 bits
    //  SDTRAS7: SDRAM tRAS Specification. Active Command delay = 7 cycles
    //  SDTRP3 : SDRAM tRP Specification. Precharge delay = 3 cycles.
    //  SDTWR2 : SDRAM tWR Specification. tWR = 2 cycles.
    //  SDTRCD3: SDRAM tRCD Specification. tRCD = 3 cycles.
    //
    //--------------------------------------------------------------------

    *pSDCTL= SDCL3|DSDCLK1|SDPSS|SDCAW8|SDRAW12|SDTRAS7|SDTRP3|SDTWR2|SDTRCD3;

    // Note that MS2 & MS3 pin multiplexed with flag2 & flag3.
    // MSEN bit must be enabled to access SDRAM, but LED7 cannot be driven with sdram
    *pSYSCTL |=MSEN;

    // Mapping Bank 2 to SDRAM
    // Make sure that jumper is set appropriately so that MS2 is connected to
    // chip select of 16-bit SDRAM device
    *pEPCTL |=B2SD;
    *pEPCTL &= ~(B0SD|B1SD|B3SD);

    //===================================================================
    //
    // Configure AMI Control Register (AMICTL0) Bank 0 for the ISSI IS61LV5128
    //
    //  WS2 : Wait States = 2 cycles
    //  HC1  : Bus Hold Cycle (at end of write access)= 1 cycle.
    //  AMIEN: Enable AMI
    //  BW8  : External Data Bus Width= 8 bits.
    //
    //--------------------------------------------------------------------

    //SRAM Settings
    *pAMICTL0 = WS2|HC1|AMIEN|BW8;

    //===================================================================
    //
    // Configure AMI Control Register (AMICTL) Bank 1 for the AMD AM29LV08
    //
    //  WS23 : Wait States= 23 cycles
    //  AMIEN: Enable AMI
    //  BW8  : External Data Bus Width= 8 bits.
    //
    //--------------------------------------------------------------------

    //Flash Settings
    *pAMICTL1 = WS23|AMIEN|BW8;

}

void init_SRU()
{

	//CS
	SRU(HIGH, DPI_PB06_I);
	SRU(HIGH, DPI_PBEN06_I);
	
	//CLK
	SRU(SPI_CLK_O, DPI_PB03_I);
	SRU(SPI_CLK_PBEN_O, DPI_PBEN03_I);

	//MOSI
	SRU(SPI_MOSI_O, DPI_PB11_I);
	SRU(HIGH, DPI_PBEN11_I);

	//MISO
	SRU(DPI_PB12_O, SPI_MISO_I);
	SRU(LOW, DPI_PBEN12_I);

	//trigerring flag
	SRU(LOW, DAI_PB10_I);
	SRU(HIGH, PBEN10_I);


	//Make other chip selects connected to the SPI FLASH high
	SRU(HIGH, DPI_PB04_I);
	SRU(HIGH, DPI_PBEN04_I);
	SRU(HIGH, DPI_PB05_I);
	SRU(HIGH, DPI_PBEN05_I);

}

// Function which in turn calls appropriate FAT functions based on the command
void CommandInterpreter(command_fat)
{
    switch(command_fat)
    {
        case 1:	// List directories and files (at current working directory)
        		ListDirectory();
    			break;
        
        case 2:	// Change directory (to a folder specified by user)
        		ChangeDirectory(string);
        		break;
        
        case 3:	// Go to a directory one level up (like  cd .. in Unix)
        		TraverseUp();
        		break;
        
        case 4:	// Go to ROOT directory
        		GotoRoot();
        		printf("\nCurrent Working Directory is: ");   
    			printpath();
        		break;
        		
        case 5:	// Get current working directory (like pwd in Unix)
        	 	printf("\nCurrent Working Directory is: ");   
    			printpath();
    			break;
        
		case 6:	// Search for a file or Directory
				Search(string);
				break;

        case 7:	// Create directory
        		CreateDirectory(string);
    			break;
        
        case 8: // Copy file (from PC/DSP to disk)
        		CopyFile(string,string1);
        		break;
        
        case 9: // Read file (from disk to PC/DSP)
        		ReadFile(string,string1);
        		break;

        case 10:	// Erase Directory
        		Erase(string);
        		break;
        
		default:	// Invalid input
        			printf("Invalid input\n\n");	
        			break;
    }
}

/**

COMMON NOTE FOR ALL THE FUNCTIONS BELOW::

	All these functions should first check if the disk is formatted or not. If not, disk should 
	be formatted. Also each of these functions should get a fresh copy of FAT entries each time
	before performing any further operation.

**/

// List Directories and Files at the current working directory
void ListDirectory()
{
    printf("\n\t\tList of Directories and Files in Current Directory\n");
    printf("\t\t==================================================\n");
    
    // Add the code for listing directories and files here
    lsDIR();
        
    // Code should be written to find out the current working directory and then display all 
    // directories and files within that directory. If this function is run for the first time
    // directory structure at root level should be displayed.
    
    // Finally display the current working directory
    printf("\nCurrent Working Directory is: ");
    printpath();
    

}

// Change Directory to the folder specified
void ChangeDirectory(char* string)
{
    changeDIR(string);
    // Add the code for changing directory to folder specified here
    // Should display proper error messages and exit program in case of errors.
    // Errors like "Invalid Directory" etc.
    
    // Finally display the current working directory
    printf("\nCurrent Working Directory is: ");   
    printpath();
}

// Go to a directory that is level up the current directory
void TraverseUp(void)
{
    traverseupDIR();
    // Add the code for traversing to a upper level directory here
    // If this function is called when control is at the root level, function should return
    // without any further processing.
    
    // At the end, code should dispaly current working directory
    printf("\nCurrent Working Directory is: ");   
    printpath();
}

void GotoRoot()
{
    gotoROOTDIR();
}

// Search for a File or Directory
void Search(char* string)
{
    searchFILE(string);
}

// Create Directory
void CreateDirectory(char* string)
{
    int temp=createDIR(string);
    // Add code for creating directory here
    // FAT entries will need to be changed 
    
    if(temp==1)
		printf("\nDirectory %s created successfully\n\n",string);    
}

// Copy File
void CopyFile(char* source,char* destination)
{    
    int success=copyFILE(source,destination);
    
    if(success)
    	printf("\n\nFile %s from PC copied to the MMC as %s \n\n",source,destination);
}

// Read File
void ReadFile(char* source, char* destination)
{
    int success=readFILE(source,destination);
 	
    if(success)
    	printf("\n\nFile %s read from the MMC and copied to the local disk as %s\n\n",source,destination);    
}

// Erase Directory/File
void Erase(char* string)
{
    erase_FILE_DIR(string);
    // Add code for erasing the directory specified. Should display message in case of invalid
    // directory etc.
    // FAT entries will need to be changed
}
