#include "mmc_access.h"
extern ext_access;
// Buffers to store commands and responses
int command_buf[9] ; 
int response_buf[9];

// Count to keep track of the interrupts
extern int interrupt_count ;

// Flag to indicate that 80 cycles needed for initialization are 
// complete (SPI DMA interrupt)
extern volatile int done_80_cycles ;

// Flag to indicate that data block is received using DMA
extern volatile int data_block_received ;

// Flag to indicate that data block is transmitted using DMA
extern volatile int data_block_transmitted ;

// Response value from the MMC Card.
int response;

// Flag used to indicate that token value (0xFE) is received.
extern int token_flag ;

// Flag used to indicate that data block is written to MMC successfully
extern volatile int data_response_flag ;

// Command that is being sent to MMC
volatile int command = 0;

// Function to Initialize MMC Card
// This includes initial 80 cycles, CMD0 and CMD1
void mmc_init()
{
	int i;
	
	// Initialize SPI for sending 80 or more dummy cycles.
	init_spi();

	// Interrupt dispatcher for SPI Core Rx ISR
	interrupts (SIG_SPIH, command_isr);

	/***************************************************************
	
				 Send Command0 - CMD_GO_IDLE_STATE
	
	****************************************************************/
	response = 0xff;
	// Continuously send the same command until the card sends a
	// valid response (i.e Card in IDLE State)
	while(response != R1_IN_IDLE_STATE)
	{
		// Initially CS is made logic "HI".
		SRU(HIGH, DPI_PB06_I);
	
		// Enable SPI in Core mode
		*pSPICTL = WL8 | MSBF | SPIMS | OPD | SMLS | GM | CPHASE | CLKPL ;

		mmc_send_cmd(CMD_GO_IDLE_STATE,0);
		response = mmc_get_response();
	}

	/***************************************************************
	
				 Send Command1 - CMD_SEND_OP_COND
	
	****************************************************************/
	response = 0xff;
	// Continuously send the same command until the card comes out
	// of idle state
	while(response != 0)
	{
		// Initially CS is made logic "HI".
		SRU(HIGH, DPI_PB06_I);
		
		// Enable SPI in Core mode
		*pSPICTL = WL8 | MSBF | SPIMS | OPD | SMLS | GM | CPHASE | CLKPL ;

		mmc_send_cmd(CMD_SEND_OP_COND,0);
		response = mmc_get_response();
	}
}

// Function to send dummy 80 cycles for initializing MMC Card
void init_spi()
{
	int i;
	int dummy[11];

	// Fill up dummy buffer with 0xff
	for(i=0;i<11;i++)
		dummy[i] = 0xff;
	
	// Initially CS is made logic "HI".
	SRU(HIGH, DPI_PB06_I);
	
	// Enable SPI DMA interrupt
	interrupts (SIG_SPIH, mmc_init_isr);
	
	// Clear SPI Control Registers
	*pSPICTL = 0x0;
	*pSPIDMAC = 0x0;
	
	// SPI Baud rate should be less than 400 KHz during the 
	// initialization process
	*pSPIBAUD = 400; // BAUDR = 200 => SPI Baud Rate = 331.776 / (8*200) = 207.36 KHz
	
	// Initialize DMA registers
	*pIISPI = (int)dummy;
	*pIMSPI = 1;
	*pCSPI = 11;
	
	

	// Program Control Registers
	*pSPICTL = SPIEN | SENDZ | SPIMS | MSBF | TIMOD2 | WL8 | CPHASE | SMLS | OPD | CLKPL; 	
	*pSPIDMAC = SPIDEN | INTEN ;
	
	// Wait for SPI DMA interrupt
	while(done_80_cycles ==0)
	{
	}
}

// Function to send commands to the MMC Card
void mmc_send_cmd(int cmd, int arg)
{
	command = cmd;
	// Filling up the command buffer
	command_buf[0] = 0x40 | cmd;
	command_buf[1] = (arg & 0xff000000) >> 24;
	command_buf[2] = (arg & 0xff0000) >> 16;
	command_buf[3] = (arg & 0xff00) >> 8;
	command_buf[4] = arg & 0xff;
	
	// CMD0 should be 0x40 00 00 00 00 95
	if(command == 0)
		command_buf[5] = 0x95;
	else
		command_buf[5] = 0x01;
		
	command_buf[6] = 0xff;
	command_buf[7] = 0xff;
	command_buf[8] = 0xff;
		
	// Enable SPI	
	*pSPICTL |= SPIEN ;

	// Assert Chip Select logic "LO" to select MMC Card.
	// Asserting Chip select logic "LO" during CMD0 will configure the
	// MMC card in SPI mode.
	SRU(LOW, DPI_PB06_I);

	// Load TX buffer with first byte
	*pTXSPI = command_buf[0];
}

// Function to return the response from the MMC Card
int mmc_get_response()
{
	// Wait till the MMC Card sends its response
	while(interrupt_count<9)
	{
	}
		
	// Clear the interrupt counter
	interrupt_count = 0 ;
	return response_buf[7];
}

// Function to set the block length
void mmc_set_block_length(int length)
{
	// Interrupt dispatcher for SPI Core Rx ISR
	interrupts (SIG_SPIH, command_isr);

	/***************************************************************
	
				 Send Command16 - CMD_SET_BLOCKLEN
	
	****************************************************************/
	response = 0xff;
	
	// Initially CS is made logic "HI".
	SRU(HIGH, DPI_PB06_I);
	
	// Enable SPI in Core mode
	*pSPICTL = WL8 | MSBF | SPIMS | OPD | SMLS | GM | CPHASE | CLKPL ;

	mmc_send_cmd(CMD_SET_BLOCKLEN,512);
	response = mmc_get_response();
	if(response!=0)
	{
		printf("Set Block Length command not successful\n");
		printf("Response from MMC: 0x%x",response);
		exit(1);
	}
}

// Function to read data from MMC Card
void mmc_read_data(int address, int length, int *destination)
{
	
    *pSPICTL = TXFLSH | RXFLSH ; 
	*pSPIDMAC = FIFOFLSH;
    // Initially CS is made logic "HI".
	SRU(HIGH, DPI_PB06_I);
	
	// Interrupt dispatcher for SPI Core Rx ISR
	interrupts (SIG_SPIH, command_isr);

	/***************************************************************
	
				 Send Command17 - CMD_READ_BLOCK
	
	****************************************************************/
	response = 0xff;

	// Enable SPI in Core mode
	*pSPICTL = WL8 | MSBF | SPIMS | OPD | SMLS | GM | CPHASE | CLKPL ;

	mmc_send_cmd(CMD_READ_BLOCK,address);
	response = mmc_get_response();
	
	if(response!=0)
	{
		printf("Read Block command not successful\n");
		printf("Response from MMC: 0x%x",response);
		exit(1);
	}
	
	// Initially CS is made logic "HI".
	SRU(HIGH, DPI_PB06_I);
	
	// Enable SPI in Core mode
	// Wait till the DSP receives the token word
	interrupts (SIG_SPIH, wait_for_token);
	*pSPICTL = WL8 | MSBF | SPIMS | OPD | SMLS | GM | CPHASE | CLKPL | SPIEN;
	SRU(LOW, DPI_PB06_I);
	*pTXSPI = 0xff;
	while(token_flag == 0)
	{
	}
	token_flag=0;
	
	
	
	SRU(HIGH, DPI_PB06_I);
	
	// Clear SPI DMA Control Register
	*pSPIDMAC = 0x0;

	// Enable SPI Rx DMA interrupt for block reception
	interrupts (SIG_SPIH, read_block_isr);
	
	// Initialize DMA parameters
	*pIISPI = (int)destination;
	*pIMSPI = 1;
	*pCSPI = length + 2 ;

	SRU(LOW, DPI_PB06_I);
	
	// Program Control Registers
	*pSPICTL = SPIEN | SPIMS | MSBF | TIMOD2 | WL8 | CPHASE | SMLS | OPD | CLKPL | GM; 	
	*pSPIDMAC = SPIDEN | INTEN | SPIRCV ;
	*pTXSPI = 0xff;
	// Wait for SPI DMA interrupt
	while(data_block_received ==0)
	{
	}
	data_block_received = 0;//m
	SRU(HIGH, DPI_PB06_I);//m
}

// Function to write data to MMC
void mmc_write_data(int address, int *source)
{
	
    *pSPICTL = TXFLSH | RXFLSH ; 
	*pSPIDMAC = FIFOFLSH;
    int* source1;
    int i=0;
    source1 = (int *)malloc(515 * sizeof(int));
	
	// Initialize source data 
	// Write block length is fixed to 512
	// First word is data token and last two words are CRC bytes
	for(i=0;i<515;i++)
	{
		// First value should be 0xFE - DATA Token
		if (i==0)
		{
			*source1 = 0xFE;
		}
		// Last two values should be 0xFF
		else if ((i==513)||(i==514))
		{
			*(source1+i) = 0xFF;
		}
		else
			*(source1+i) = source[i-1] & 0xFF; // User specific data
	}
	
    
    // Initially CS is made logic "HI".
	SRU(HIGH, DPI_PB06_I);

	// Interrupt dispatcher for SPI Core Rx ISR
	interrupts (SIG_SPIH, command_isr);

	/***************************************************************
	
				 Send Command24 - CMD_WRITE_BLOCK
	
	****************************************************************/
	
	response = 0xff;

	// Enable SPI in Core mode
	*pSPICTL = WL8 | MSBF | SPIMS | OPD | SMLS | GM | CPHASE | CLKPL ;

	mmc_send_cmd(CMD_WRITE_BLOCK,address);
	response = mmc_get_response();
	
	if(response!=0)
	{
		printf("Write command not successful\n");
		printf("Response from MMC: 0x%x",response);
		exit(1);
	}
	
	// Initially CS is made logic "HI".
	SRU(HIGH, DPI_PB06_I);
	
	// Clear SPI DMA Control Register
	*pSPIDMAC = 0x0;
	
	// Enable SPI Tx DMA interrupt
	interrupts (SIG_SPIH, write_block_isr);
	
	SRU(LOW, DPI_PB06_I);

	// Initialize DMA parameters
	*pIISPI = (int)source1;
	*pIMSPI = 1;
	*pCSPI = 515 ;

	// Program Control Registers
	*pSPICTL = SPIEN | SPIMS | MSBF | TIMOD2 | WL8 | CPHASE | SMLS | OPD | CLKPL | GM; 	
	*pSPIDMAC = SPIDEN | INTEN ;
	
	// Wait for SPI DMA interrupt
	while(data_block_transmitted ==0)
	{
	}	
	data_block_transmitted =0;//m
	SRU(HIGH, DPI_PB06_I);//m

	// Wait till the data is written sucessfully into the 
	// MMC Card
	interrupts (SIG_SPIH, wait_for_data_response);
	*pSPICTL = WL8 | MSBF | SPIMS | OPD | SMLS | GM | CPHASE | CLKPL | SPIEN;
	SRU(LOW, DPI_PB06_I);
	*pTXSPI = 0xff;
	
	while(data_response_flag == 0)
	{
	}
	data_response_flag = 0;
	free(source1);

	
	SRU(HIGH, DPI_PB06_I);
}

// Function to turn the CRC check on or off
// Pass '1' for ON and '0' for OFF
void mmc_crc_on_off(int on_off)
{
	// Interrupt dispatcher for SPI Core Rx ISR
	interrupts (SIG_SPIH, command_isr);

	/***************************************************************
	
				 Send Command59 - CMD_CRC_ONOFF
	
	****************************************************************/
	response = 0xff;
	
	// Initially CS is made logic "HI".
	SRU(HIGH, DPI_PB06_I);
	
	// Enable SPI in Core mode
	*pSPICTL = WL8 | MSBF | SPIMS | OPD | SMLS | GM | CPHASE | CLKPL ;

	mmc_send_cmd(CMD_CRC_ONOFF,on_off);
	response = mmc_get_response();
	if(response!=0)
	{
		printf("Set CRC ON/OFF command not successful\n");
		printf("Response from MMC: 0x%x",response);
		exit(1);
	}
}

void read_BufferSector(BYTE* buff,DWORD sector_offset,int count)
{
    int i=0;
    
    DWORD start_add=sector_offset*512;
    
   	
   	for(i=0;i<count;i++)
   	{
   	        mmc_read_data(start_add+i*512,512,read_buff);
   	        int j=0;
   	        for(j=0;j<512;j++)
   	        {
   	            int index=i*512+j;
   	            buff[index]=read_buff[j];
   	        }
   	        
   	  	 
   	}
   	
}

void write_BufferSector(BYTE* buff,DWORD sector_offset,int count)
{
    int i=0;
    
    DWORD start_add=sector_offset*512;
    
   	for(i=0;i<count;i++)
   	{
   	 
   	    int j=0;
   	    for(j=0;j<512;j++)
   	    {
   	       int index=i*512+j;
   	       write_buff[j]=buff[index];
   	    }
   	    mmc_write_data(start_add+i*512,write_buff);
   	    
   	    
   	    
   	}
}

void read_SectorData(WORD sublocation, BYTE *buffer, WORD size)
{
	WORD i;
	for(i=0;i<size;i++)
	{
		buffer[i]=buffers.currentsector.u8data[sublocation+i];
	}
}

void write_SectorData(WORD sublocation, BYTE *buffer, WORD size)
{
	WORD i;
	for(i=0;i<size;i++)
	{
		buffers.currentsector.u8data[sublocation+i]=buffer[i];
	}
}

BYTE read_BYTE(WORD sublocation)
{
	BYTE data=0;          
	
	    	data=buffers.currentsector.u8data[sublocation];	  
	
	return data;
}

WORD read_WORD(WORD sublocation) 
{
	WORD data=0; 
	 
	data	=buffers.currentsector.u8data[sublocation];	  
	data   +=buffers.currentsector.u8data[sublocation+1]<<8;
	return data;					  			 
}

DWORD read_DWORD(WORD sublocation) 
{
	DWORD data=0; 
	 
	data	=buffers.currentsector.u8data[sublocation];	  
	data   +=buffers.currentsector.u8data[sublocation+1]<<8;
	data   +=buffers.currentsector.u8data[sublocation+2]<<16;
	data   +=buffers.currentsector.u8data[sublocation+3]<<24;
	return data;	
}

void write_BYTE(WORD sublocation, BYTE value)
{
    buffers.currentsector.u8data[sublocation]=value;
}

void write_WORD(WORD sublocation, WORD value)
{
    buffers.currentsector.u8data[sublocation]=value&0x0000000F;
    buffers.currentsector.u8data[sublocation+1]=(value&0x000000F0)>>8;
}
